'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var deepmerge = _interopDefault(require('deepmerge'));

var minBreakpoint = function minBreakpoint(breakpoint) {
  return breakpoint !== 0 ? breakpoint : null;
};
var minWidth = function minWidth(value) {
  return "@media screen and (min-width: " + value + ")";
};
var DEFAULT_BREAKPOINTS =
/*#__PURE__*/
[0, 40, 52, 64].map(function (n) {
  return n > 0 ? n + 'em' : n;
});

var is = function is(n) {
  return n !== undefined && n !== null;
};
var num = function num(n) {
  return typeof n === 'number' && !Number.isNaN(n);
};
var string = function string(n) {
  return typeof n === 'string' && n !== '';
};
var obj = function obj(n) {
  return typeof n === 'object' && n !== null;
};
var func = function func(n) {
  return typeof n === 'function';
};
var negative = function negative(n) {
  return n < 0;
};
var get = function get(obj, path) {
  return String(path).split('.').reduce(function (a, b) {
    return a && is(a[b]) ? a[b] : undefined;
  }, obj);
};
function merge(acc, item) {
  if (!item) {
    return acc;
  }

  return deepmerge(acc, item, {
    clone: false // No need to clone deep, it's way faster.

  });
}

function callOrReturn(fn, arg) {
  if (!func(fn)) return fn;
  var next = fn(arg);
  return callOrReturn(next, arg);
}

function getThemeValue(theme, path, initial) {
  if (!theme) return undefined;
  return callOrReturn(get(initial || theme, path), {
    theme: theme
  });
}

function getValue(value, variants, theme) {
  if (is(variants)) {
    var valueFromVariants = getThemeValue(theme, value, variants);

    if (is(valueFromVariants)) {
      return valueFromVariants;
    }
  }

  return value;
}

function styleFromValue(cssProperties, value, theme, key, transformValue) {
  var variants = getThemeValue(theme, key);
  var computedValue = getValue(value, variants, theme);

  if (string(computedValue) || num(computedValue)) {
    var _style = {};

    for (var i = 0; i < cssProperties.length; i++) {
      _style[cssProperties[i]] = transformValue ? transformValue(computedValue, {
        rawValue: value,
        variants: variants
      }) : computedValue;
    }

    return _style;
  }

  return null;
}

function getBreakpoints(theme) {
  var themeBreakpoints = getThemeValue(theme, 'breakpoints');

  if (is(themeBreakpoints)) {
    return themeBreakpoints;
  }

  return DEFAULT_BREAKPOINTS;
}

function createStyleGenerator(getStyle, props, generators) {
  var getStyles = function getStyles(props) {
    var theme = props.theme || null;
    return getStyle(props, theme);
  };

  getStyles.meta = {
    props: props,
    getStyle: getStyle,
    generators: generators
  };
  return getStyles;
}

function styleFromBreakPoint(cssProperties, value, theme, key, transformValue) {
  var breakpoints = getBreakpoints(theme);
  var keys = Object.keys(value);
  var allStyle = {};

  for (var i = 0; i < keys.length; i++) {
    var breakpoint = keys[i];

    var _style2 = styleFromValue(cssProperties, value[breakpoint], theme, key, transformValue);

    if (_style2 !== null) {
      var breakpointValue = minBreakpoint(breakpoints[breakpoint]);

      if (breakpointValue === null) {
        allStyle = merge(allStyle, _style2);
      } else if (breakpointValue) {
        var _merge;

        allStyle = merge(allStyle, (_merge = {}, _merge[minWidth(breakpointValue)] = _style2, _merge));
      }
    }
  }

  return allStyle;
}

function getStyleFactory(prop, cssProperties, key, transformValue) {
  return function getStyle(attrs, theme) {
    var value = attrs[prop];
    if (!is(value)) return null;
    var style = styleFromValue(cssProperties, value, theme, key, transformValue);

    if (style !== null) {
      return style;
    }

    if (obj(value)) {
      return styleFromBreakPoint(cssProperties, value, theme, key, transformValue);
    }

    return null;
  };
}

function style(_ref) {
  var prop = _ref.prop,
      cssProperties = _ref.cssProperties,
      cssProperty = _ref.cssProperty,
      alias = _ref.alias,
      _ref$key = _ref.key,
      key = _ref$key === void 0 ? null : _ref$key,
      _ref$transformValue = _ref.transformValue,
      transformValue = _ref$transformValue === void 0 ? null : _ref$transformValue;
  cssProperties = cssProperties || (cssProperty ? [cssProperty] : [prop]);
  var getStyle = getStyleFactory(prop, cssProperties, key, transformValue);
  var generator = createStyleGenerator(getStyle, [prop]);

  if (!alias) {
    return generator;
  }

  return compose(generator, style({
    prop: alias,
    cssProperties: cssProperties,
    key: key,
    transformValue: transformValue
  }));
}

function indexGeneratorsByProp(styles) {
  var index = {};

  for (var i = 0; i < styles.length; i++) {
    var _style3 = styles[i];

    if (_style3 && _style3.meta) {
      var propsKeys = Object.keys(_style3.meta.props);

      for (var j = 0; j < propsKeys.length; j++) {
        var prop = _style3.meta.props[propsKeys[j]];
        index[prop] = _style3;
      }
    }
  }

  return index;
}

function compose() {
  var flatGenerators = [];

  for (var _len = arguments.length, generators = new Array(_len), _key = 0; _key < _len; _key++) {
    generators[_key] = arguments[_key];
  }

  generators.forEach(function (gen) {
    if (gen.meta.generators) {
      flatGenerators = [].concat(flatGenerators, gen.meta.generators);
    } else {
      flatGenerators.push(gen);
    }
  });
  var generatorsByProp = indexGeneratorsByProp(flatGenerators);

  function getStyle(attrs, theme) {
    var propKeys = Object.keys(attrs);
    var propCount = propKeys.length;
    var allStyle = {};

    for (var i = 0; i < propCount; i++) {
      var propKey = propKeys[i];
      var generator = generatorsByProp[propKey];

      if (generator) {
        allStyle = merge(allStyle, generator.meta.getStyle(attrs, theme));
      }
    }

    return allStyle;
  }

  var props = flatGenerators.reduce(function (keys, generator) {
    return [].concat(keys, generator.meta.props);
  }, []);
  return createStyleGenerator(getStyle, props, generators);
}

var background =
/*#__PURE__*/
style({
  prop: 'background'
});
var backgroundColor =
/*#__PURE__*/
style({
  prop: 'backgroundColor',
  alias: 'bg',
  key: 'colors'
});
var backgroundImage =
/*#__PURE__*/
style({
  prop: 'backgroundImage'
});
var backgroundSize =
/*#__PURE__*/
style({
  prop: 'backgroundSize'
});
var backgroundPosition =
/*#__PURE__*/
style({
  prop: 'backgroundPosition'
});
var backgroundRepeat =
/*#__PURE__*/
style({
  prop: 'backgroundRepeat'
});
var backgrounds =
/*#__PURE__*/
compose(background, backgroundColor, backgroundImage, backgroundSize, backgroundPosition, backgroundRepeat);

var opacity =
/*#__PURE__*/
style({
  prop: 'opacity'
});
var overflow =
/*#__PURE__*/
style({
  prop: 'overflow'
});
var basics =
/*#__PURE__*/
compose(opacity, overflow);

var DEFAULT_SPACE = [0, 4, 8, 16, 32, 64, 128, 256, 512];
var unit = function unit(_unit) {
  return function (value) {
    return num(value) ? "" + value + _unit : value;
  };
};
var px =
/*#__PURE__*/
unit('px');
var percent = function percent(n) {
  return !num(n) || n > 1 ? px(n) : n * 100 + "%";
};
var scale = function scale(defaultVariants) {
  if (defaultVariants === void 0) {
    defaultVariants = DEFAULT_SPACE;
  }

  return function (transformedValue, _ref) {
    var rawValue = _ref.rawValue,
        _ref$variants = _ref.variants,
        variants = _ref$variants === void 0 ? defaultVariants : _ref$variants;

    if (!num(rawValue)) {
      return variants[rawValue] || rawValue;
    }

    var abs = Math.abs(rawValue);
    var neg = negative(rawValue);
    var value = variants[abs] || abs;

    if (!num(value)) {
      return neg ? "-" + value : value;
    }

    return value * (neg ? -1 : 1);
  };
};

var getBorder = function getBorder(n) {
  return num(n) && n > 0 ? n + "px solid" : n;
};

var border =
/*#__PURE__*/
style({
  prop: 'border',
  key: 'borders',
  transformValue: getBorder
});
var borderWidth =
/*#__PURE__*/
style({
  prop: 'borderWidth',
  key: 'borderWidths',
  transformValue: px
});
var borderStyle =
/*#__PURE__*/
style({
  prop: 'borderStyle',
  key: 'borderStyles'
});
var borderColor =
/*#__PURE__*/
style({
  prop: 'borderColor',
  key: 'colors'
});
var borderRadius =
/*#__PURE__*/
style({
  prop: 'borderRadius',
  key: 'radii',
  transformValue: px
});
var borderTop =
/*#__PURE__*/
style({
  prop: 'borderTop',
  key: 'borders',
  transformValue: getBorder
});
var borderTopColor =
/*#__PURE__*/
style({
  prop: 'borderTopColor',
  key: 'colors'
});
var borderRight =
/*#__PURE__*/
style({
  prop: 'borderRight',
  key: 'borders',
  transformValue: getBorder
});
var borderRightColor =
/*#__PURE__*/
style({
  prop: 'borderRightColor',
  key: 'colors'
});
var borderBottom =
/*#__PURE__*/
style({
  prop: 'borderBottom',
  key: 'borders',
  transformValue: getBorder
});
var borderBottomColor =
/*#__PURE__*/
style({
  prop: 'borderBottomColor',
  key: 'colors'
});
var borderLeft =
/*#__PURE__*/
style({
  prop: 'borderLeft',
  key: 'borders',
  transformValue: getBorder
});
var borderLeftColor =
/*#__PURE__*/
style({
  prop: 'borderLeftColor',
  key: 'colors'
});
var boxShadow =
/*#__PURE__*/
style({
  prop: 'boxShadow',
  key: 'shadows'
});
var borders =
/*#__PURE__*/
compose(border, borderWidth, borderStyle, borderColor, borderTop, borderTopColor, borderRight, borderRightColor, borderBottom, borderBottomColor, borderLeft, borderLeftColor, borderRadius, boxShadow);

var width =
/*#__PURE__*/
style({
  prop: 'width',
  transformValue: percent,
  key: 'widths'
});
var height =
/*#__PURE__*/
style({
  prop: 'height',
  transformValue: percent,
  key: 'heights'
});
var size =
/*#__PURE__*/
style({
  prop: 'size',
  cssProperties: ['width', 'height'],
  transformValue: percent,
  key: 'sizes'
});
var maxWidth =
/*#__PURE__*/
style({
  prop: 'maxWidth',
  transformValue: percent,
  key: 'widths'
});
var maxHeight =
/*#__PURE__*/
style({
  prop: 'maxHeight',
  transformValue: percent,
  key: 'heights'
});
var minWidth$1 =
/*#__PURE__*/
style({
  prop: 'minWidth',
  transformValue: percent,
  key: 'widths'
});
var minHeight =
/*#__PURE__*/
style({
  prop: 'minHeight',
  transformValue: percent,
  key: 'heights'
});
var dimensions =
/*#__PURE__*/
compose(width, height, maxWidth, maxHeight, minWidth$1, minHeight);

var display =
/*#__PURE__*/
style({
  prop: 'display'
});
var alignItems =
/*#__PURE__*/
style({
  prop: 'alignItems'
});
var alignContent =
/*#__PURE__*/
style({
  prop: 'alignContent'
});
var justifyItems =
/*#__PURE__*/
style({
  prop: 'justifyItems'
});
var justifyContent =
/*#__PURE__*/
style({
  prop: 'justifyContent'
});
var flexWrap =
/*#__PURE__*/
style({
  prop: 'flexWrap'
});
var flexBasis =
/*#__PURE__*/
style({
  prop: 'flexBasis',
  transformValue: percent
});
var flexDirection =
/*#__PURE__*/
style({
  prop: 'flexDirection'
});
var flex =
/*#__PURE__*/
style({
  prop: 'flex'
});
var justifySelf =
/*#__PURE__*/
style({
  prop: 'justifySelf'
});
var alignSelf =
/*#__PURE__*/
style({
  prop: 'alignSelf'
});
var order =
/*#__PURE__*/
style({
  prop: 'order'
});
var flexboxes =
/*#__PURE__*/
compose(display, alignItems, alignContent, justifyContent, flexWrap, flexBasis, flexDirection, flex, justifySelf, alignSelf, order);

var gridGap =
/*#__PURE__*/
style({
  prop: 'gridGap',
  key: 'space',
  transformValue:
  /*#__PURE__*/
  scale()
});
var gridColumnGap =
/*#__PURE__*/
style({
  prop: 'gridColumnGap',
  key: 'space',
  transformValue:
  /*#__PURE__*/
  scale()
});
var gridRowGap =
/*#__PURE__*/
style({
  prop: 'gridRowGap',
  key: 'space',
  transformValue:
  /*#__PURE__*/
  scale()
});
var gridColumn =
/*#__PURE__*/
style({
  prop: 'gridColumn'
});
var gridRow =
/*#__PURE__*/
style({
  prop: 'gridRow'
});
var gridAutoFlow =
/*#__PURE__*/
style({
  prop: 'gridAutoFlow'
});
var gridAutoColumns =
/*#__PURE__*/
style({
  prop: 'gridAutoColumns'
});
var gridAutoRows =
/*#__PURE__*/
style({
  prop: 'gridAutoRows'
});
var gridTemplateColumns =
/*#__PURE__*/
style({
  prop: 'gridTemplateColumns'
});
var gridTemplateRows =
/*#__PURE__*/
style({
  prop: 'gridTemplateRows'
});
var gridTemplateAreas =
/*#__PURE__*/
style({
  prop: 'gridTemplateAreas'
});
var gridArea =
/*#__PURE__*/
style({
  prop: 'gridArea'
});
var grids =
/*#__PURE__*/
compose(gridGap, gridColumnGap, gridRowGap, gridColumn, gridRow, gridAutoFlow, gridAutoColumns, gridAutoRows, gridTemplateColumns, gridTemplateRows, gridTemplateAreas, gridArea);

var position =
/*#__PURE__*/
style({
  prop: 'position'
});
var zIndex =
/*#__PURE__*/
style({
  prop: 'zIndex',
  key: 'zIndices'
});
var top =
/*#__PURE__*/
style({
  prop: 'top',
  transformValue: px
});
var right =
/*#__PURE__*/
style({
  prop: 'right',
  transformValue: px
});
var bottom =
/*#__PURE__*/
style({
  prop: 'bottom',
  transformValue: px
});
var left =
/*#__PURE__*/
style({
  prop: 'left',
  transformValue: px
});
var positions =
/*#__PURE__*/
compose(position, zIndex, top, right, bottom, left);

var key = 'space';
var margin =
/*#__PURE__*/
style({
  prop: 'margin',
  alias: 'm',
  cssProperties: ['margin'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var marginTop =
/*#__PURE__*/
style({
  prop: 'marginTop',
  alias: 'mt',
  cssProperties: ['marginTop'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var marginRight =
/*#__PURE__*/
style({
  prop: 'marginRight',
  alias: 'mr',
  cssProperties: ['marginRight'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var marginBottom =
/*#__PURE__*/
style({
  prop: 'marginBottom',
  alias: 'mb',
  cssProperties: ['marginBottom'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var marginLeft =
/*#__PURE__*/
style({
  prop: 'marginLeft',
  alias: 'ml',
  cssProperties: ['marginLeft'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var mx =
/*#__PURE__*/
style({
  prop: 'mx',
  cssProperties: ['marginRight', 'marginLeft'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var my =
/*#__PURE__*/
style({
  prop: 'my',
  cssProperties: ['marginTop', 'marginBottom'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var padding =
/*#__PURE__*/
style({
  prop: 'padding',
  alias: 'p',
  cssProperties: ['padding'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var paddingTop =
/*#__PURE__*/
style({
  prop: 'paddingTop',
  alias: 'pt',
  cssProperties: ['paddingTop'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var paddingRight =
/*#__PURE__*/
style({
  prop: 'paddingRight',
  alias: 'pr',
  cssProperties: ['paddingRight'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var paddingBottom =
/*#__PURE__*/
style({
  prop: 'paddingBottom',
  alias: 'pb',
  cssProperties: ['paddingBottom'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var paddingLeft =
/*#__PURE__*/
style({
  prop: 'paddingLeft',
  alias: 'pl',
  cssProperties: ['paddingLeft'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var px$1 =
/*#__PURE__*/
style({
  prop: 'px',
  cssProperties: ['paddingRight', 'paddingLeft'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var py =
/*#__PURE__*/
style({
  prop: 'py',
  cssProperties: ['paddingTop', 'paddingBottom'],
  key: key,
  transformValue:
  /*#__PURE__*/
  scale()
});
var space =
/*#__PURE__*/
compose(margin, marginTop, marginRight, marginBottom, marginLeft, mx, my, padding, paddingTop, paddingRight, paddingBottom, paddingLeft, px$1, py);

var fontFamily =
/*#__PURE__*/
style({
  prop: 'fontFamily',
  key: 'fonts'
});
var fontSize =
/*#__PURE__*/
style({
  prop: 'fontSize',
  key: 'fontSizes',
  transformValue:
  /*#__PURE__*/
  scale([12, 14, 16, 20, 24, 32, 48, 64, 72])
});
var lineHeight =
/*#__PURE__*/
style({
  prop: 'lineHeight',
  key: 'lineHeights'
});
var fontWeight =
/*#__PURE__*/
style({
  prop: 'fontWeight',
  key: 'fontWeights'
});
var textAlign =
/*#__PURE__*/
style({
  prop: 'textAlign'
});
var letterSpacing =
/*#__PURE__*/
style({
  prop: 'letterSpacing',
  key: 'letterSpacings',
  transformValue: px
});
var textColor =
/*#__PURE__*/
style({
  prop: 'color',
  key: 'colors'
});
var textTransform =
/*#__PURE__*/
style({
  prop: 'textTransform'
});
var typography =
/*#__PURE__*/
compose(fontFamily, fontSize, lineHeight, fontWeight, textAlign, letterSpacing, textColor, textTransform);

var color =
/*#__PURE__*/
compose(backgroundColor, textColor);

var system =
/*#__PURE__*/
compose(backgrounds, basics, borders, dimensions, flexboxes, grids, positions, space, typography);

var variant = function variant(_ref) {
  var key = _ref.key,
      _ref$prop = _ref.prop,
      prop = _ref$prop === void 0 ? 'variant' : _ref$prop;
  return function (props) {
    return get(props.theme, [key, props[prop]].join('.')) || null;
  };
};

var buttonStyle =
/*#__PURE__*/
variant({
  key: 'buttons'
});
var textStyle =
/*#__PURE__*/
variant({
  key: 'textStyles',
  prop: 'textStyle'
});
var colorStyle =
/*#__PURE__*/
variant({
  key: 'colorStyles',
  prop: 'colors'
});

exports.alignContent = alignContent;
exports.alignItems = alignItems;
exports.alignSelf = alignSelf;
exports.background = background;
exports.backgroundColor = backgroundColor;
exports.backgroundImage = backgroundImage;
exports.backgroundPosition = backgroundPosition;
exports.backgroundRepeat = backgroundRepeat;
exports.backgroundSize = backgroundSize;
exports.backgrounds = backgrounds;
exports.basics = basics;
exports.border = border;
exports.borderBottom = borderBottom;
exports.borderBottomColor = borderBottomColor;
exports.borderColor = borderColor;
exports.borderLeft = borderLeft;
exports.borderLeftColor = borderLeftColor;
exports.borderRadius = borderRadius;
exports.borderRight = borderRight;
exports.borderRightColor = borderRightColor;
exports.borderStyle = borderStyle;
exports.borderTop = borderTop;
exports.borderTopColor = borderTopColor;
exports.borderWidth = borderWidth;
exports.borders = borders;
exports.bottom = bottom;
exports.boxShadow = boxShadow;
exports.buttonStyle = buttonStyle;
exports.color = color;
exports.colorStyle = colorStyle;
exports.compose = compose;
exports.dimensions = dimensions;
exports.display = display;
exports.flex = flex;
exports.flexBasis = flexBasis;
exports.flexDirection = flexDirection;
exports.flexWrap = flexWrap;
exports.flexboxes = flexboxes;
exports.fontFamily = fontFamily;
exports.fontSize = fontSize;
exports.fontWeight = fontWeight;
exports.gridArea = gridArea;
exports.gridAutoColumns = gridAutoColumns;
exports.gridAutoFlow = gridAutoFlow;
exports.gridAutoRows = gridAutoRows;
exports.gridColumn = gridColumn;
exports.gridColumnGap = gridColumnGap;
exports.gridGap = gridGap;
exports.gridRow = gridRow;
exports.gridRowGap = gridRowGap;
exports.gridTemplateAreas = gridTemplateAreas;
exports.gridTemplateColumns = gridTemplateColumns;
exports.gridTemplateRows = gridTemplateRows;
exports.grids = grids;
exports.height = height;
exports.justifyContent = justifyContent;
exports.justifyItems = justifyItems;
exports.justifySelf = justifySelf;
exports.left = left;
exports.letterSpacing = letterSpacing;
exports.lineHeight = lineHeight;
exports.margin = margin;
exports.marginBottom = marginBottom;
exports.marginLeft = marginLeft;
exports.marginRight = marginRight;
exports.marginTop = marginTop;
exports.maxHeight = maxHeight;
exports.maxWidth = maxWidth;
exports.minHeight = minHeight;
exports.minWidth = minWidth$1;
exports.mx = mx;
exports.my = my;
exports.opacity = opacity;
exports.order = order;
exports.overflow = overflow;
exports.padding = padding;
exports.paddingBottom = paddingBottom;
exports.paddingLeft = paddingLeft;
exports.paddingRight = paddingRight;
exports.paddingTop = paddingTop;
exports.position = position;
exports.positions = positions;
exports.px = px$1;
exports.py = py;
exports.right = right;
exports.size = size;
exports.space = space;
exports.style = style;
exports.system = system;
exports.textAlign = textAlign;
exports.textColor = textColor;
exports.textStyle = textStyle;
exports.textTransform = textTransform;
exports.top = top;
exports.typography = typography;
exports.variant = variant;
exports.width = width;
exports.zIndex = zIndex;
